| display.describe_posterior | R Documentation |
Prints tables (i.e. data frame) in different output formats.
## S3 method for class 'describe_posterior'
display(object, format = "markdown", ...)
## S3 method for class 'describe_posterior'
print(x, digits = 2, caption = "Summary of Posterior Distribution", ...)
## S3 method for class 'describe_posterior'
print_html(x, digits = 2, caption = "Summary of Posterior Distribution", ...)
## S3 method for class 'describe_posterior'
print_md(x, digits = 2, caption = "Summary of Posterior Distribution", ...)
object, x |
An object returned by one of the package's function, for
example |
format |
String, indicating the output format. Can be |
... |
Arguments passed down to |
digits |
Integer, number of digits to round the table output. Defaults to 2. |
caption |
Character, caption for the table. If |
display() is useful when the table-output from functions, which is
usually printed as formatted text-table to console, should be formatted for
pretty table-rendering in markdown documents, or if knitted from rmarkdown
to PDF or Word files. See
vignette
for examples.
If format = "markdown", the return value will be a character
vector in markdown-table format. If format = "html", an object of
class gt_tbl. If format = "tt", an object of class tinytable.
d <- data.frame(replicate(4, rnorm(20)))
result <- describe_posterior(d)
# markdown format
display(result)
# gt HTML
display(result, format = "html")
# tinytable
display(result, format = "tt")
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.